@@ -22,7 +22,7 @@ def mkdir (file_path):
         pass
 
 dtypex = ['bigint', 'numeric', 'smallint', 'int', 'float', 'real', 'date', 'datetime', 'datetime2', 'time',
-    'varchar(250)', 'varchar(500)', 'varchar(100)', 'varchar(1500)', 'varchar(2000)' ]
+    'varchar(250)', 'varchar(500)', 'varchar(100)', 'varchar(1500)', 'varchar(2000)', 'text' ]
 
 
 class RunMain:
@@ -55,6 +55,7 @@ class RunMain:
         verbose = dargs['v']
         alias   = dargs['alias']
         delim   = dargs['delim']
+        as_null = dargs['null']
 
         column_types   = dargs['column_types']
         where_fields   = dargs['where_fields']
@@ -64,6 +65,8 @@ class RunMain:
         batchSize      = dargs['bsize']
         qstring        = dargs['q']
 
+        sh = dargs['sh']
+        sf = dargs['sf']
         # Run(self,sqlFileName, is_file=True, verbose=True, output_file="")
 
         if dargs['find_params']: #argument for first hit
@@ -100,6 +103,16 @@ class RunMain:
             f.close()
             return
 
+        if dargs['readme']:
+            in_path = pathlib.Path(__file__).parent.resolve()
+            fpath   = os.path.join(in_path, 'README.md')
+            f = open(fpath)
+            s = f.read()
+            print(s)
+            f.close()
+            return
+
+
 
         #run raw query string
         if not dargs['q'] is None:
@@ -113,7 +126,7 @@ class RunMain:
         fext = pathlib.Path(in_file_name).suffix
         if fext in ['.xls', '.xlsx', '.csv']:
             input_file_path = in_file_name
-            x = DataLoader(input_file_path, delimiter = delim, stripDateTime=stripDateTime, alias=alias  )
+            x = DataLoader(input_file_path, delimiter = delim, stripDateTime=stripDateTime, alias=alias, skipHeaderRows=sh, skipFooterRows=sf  )
             df = x.LoadFile()
             #sql script file creation:
             table_name  = table_name
@@ -121,7 +134,7 @@ class RunMain:
             sql_file = os.path.join(odir, table_name)
             csqlx  = SqlDDL(df, schema_name, table_name, crud_type,  sql_file, batchSize = batchSize,
                 set_fields = set_fields, where_fields = where_fields, 
-                column_types=column_types)
+                column_types=column_types, as_null=as_null)
             csqlx.CreateFiles()
 
             if dargs['sexe']:
@@ -149,7 +162,28 @@ class RunMain:
 
     #simple script template
     def InitializeParser(self):
-        parser = argparse.ArgumentParser(description='Process some integers.')
+
+        main_description = """
+        Module to convert xlsx/xls/csv into sql server files and return sql queries into csv/xls/xlsx. For help use the following commands:
+        To view readme: https://pypi.org/project/tsqlxlsx/
+
+        list all pyodbc drivers available:
+            python -m tsqlxlsx --ld:
+        dislpay example.bat file for running program as windows executable
+            python -m tsqlxlsx --pbat
+        display example python script for calling and running modules in python:
+            python -m tsqlxlsx --ppy
+        display README.md for module
+            python -m tsqlxlsx --readme
+
+        to output file file use
+        linux/mac
+        python -m --readme > pathToFolder/readme.md 
+        windows:
+        python -m --readme > C:\pathToFolder\readme.md 
+        """
+
+        parser = argparse.ArgumentParser(description=main_description)
         # parser.add_argument('integers', metavar='N', type=int, nargs='+',
         #                     help='an integer for the accumulator')
         findp_h =""" this flag makes serveral attempts to connect to a tsql server with different conneciton parameters. The first
@@ -165,7 +199,7 @@ class RunMain:
         parser.add_argument('--ld', help= "list all pyodbc drivers available", action='store_true')
         parser.add_argument('--pbat', help= "dislpay example.bat file for running program as windows executable", action='store_true')
         parser.add_argument('--ppy', help= "dislpay example.py file for running program as python module", action='store_true')
-
+        parser.add_argument('--readme', help= "dislpay modules readme.md markdown file", action='store_true')
 
         parser.add_argument('-s', help= 'name of the server', default= "")
         parser.add_argument('-d', help= 'name of the database.', default= "")
@@ -193,6 +227,10 @@ class RunMain:
 
 
         parser.add_argument('-f', help= 'Path to excel or csv file. Header must be present in file. if extension is .sql will run commands in the file directly')
+
+        parser.add_argument('-sh',help= "List of rows to skip starting for top. For sqlcmd a --- is usually the second line. use -sh 1 to ignore if reading from csv that was created using sqlcmd direclty or exporting to .xlsx/.xls without specifying pyodbc_driver" , nargs='+', type=int )
+        parser.add_argument('-sf',help= "Number of rows to skip starting for bottom. For sqlcmd a --- x rows affected is normally added at bottom. Use -sf 1 to ignore if reading from csv that was created using sqlcmd direclty or exporting to .xlsx/.xls without specifying pyodbc_driver" , nargs='+', type=int )
+
         parser.add_argument('-sexe', help= 'Optional. Skip execution of sql files generated from .xls, .xlsx or .csv files', action='store_true', default=False)
 
 
@@ -226,6 +264,10 @@ class RunMain:
         name of columns should be names of alias columns if used
         """, nargs="+", default = None)
 
+        parser.add_argument('-null', help= """list of column_names when missing should be set to sql server null value. Default behavior assumes value
+        is a string and replaces values with 'NULL' where Null is a string. 
+        """, nargs="+", default = None)
+
         parser.add_argument('-bsize', help="Batch size. Max number of sql commands sent in one file. 7000 appears to be the max allowed", default = 7000, type = int)
 
         help_std = """Date columns in excel are automatically converted to date time. In order to skip this effect use the -std flag. 
@@ -270,6 +312,10 @@ class RunMain:
 
         if dargs['ppy']:
             return
+
+        if dargs['readme']:
+            return
+
         #print help
         if not dargs['find_params'] and not dargs['find_params_1'] and not dargs['ld'] and not dargs['pbat'] and not dargs['ppy'] and dargs['f'] is None and dargs['q'] is None:
             raise Exception('No arguments specified use -h for help')
